#!/bin/bash
# (C) Copyright  2007-2016, 2018-2019 Hewlett Packard Enterprise Development, LP.
# Version: A.12.50.00
#
# Revision History:
# 1) Enhancement to support SLES15 (QXCR1001637315) ( 29/10/2018 )
# 2) Enhancement to support RHEL8 (QXCR1001704731) (22/08/2019) 
############################################################################

# Sourcing cluster configuration file and utils.sh as it is required
# during online reconfiguration of the package attributes.

. /etc/cmcluster.conf

if [[ -z $SG_UTILS ]]; then
    SG_UTILS=$SGCONF/scripts/mscripts/utils.sh
fi

if [[ -f ${SG_UTILS} ]]; then
   . ${SG_UTILS}
   if (( $? != 0 ))
   then
      echo "ERROR: Unable to source package utility functions file: ${SG_UTILS}"
      exit 1
   fi
else
   echo "ERROR: Unable to find package utility functions file: ${SG_UTILS}"
   exit 1
fi

# This is needed as some of the SG Package attributes are used in SG_UTILS
# script
sg_source_pkg_env $*

#Check the release version of Redhat if OS is RedHat.
#Else check whether the OS is SUSE and determine the destro version.
#Setting the default value for release as -1
release=-1

rpc_service="/usr/lib/systemd/system/rpcbind.service"
statd_service="/usr/lib/systemd/system/rpc-statd.service"
idmap_service="/usr/lib/systemd/system/nfs-idmapd.service"
nfs_service="/usr/lib/systemd/system/nfs-server.service"
mountd_service="/usr/lib/systemd/system/nfs-mountd.service"

#############  Below check is added as part of RH8 NFS Toolkit Testing ##########
################################# START #########################################
version=0
IS_SUSE=`cat /proc/version | grep -i "SuSE" | grep -v "grep"`


function check_for_os_version
{
        typeset os_version=0

        # "/etc/os-release" file exists for versions >7.0 in case of Red Hat and >12 in case of SLES.
        if [[ -f /etc/os-release ]];then
                os_version=`cat /etc/os-release | grep "VERSION_ID" | tr -d '"' | cut -d "=" -f 2`
        fi
        echo "$os_version"
}

if [[ -f /etc/os-release ]]; then

    version=$(check_for_os_version)
    release_num=${version/.*}

elif [[ -z $IS_SUSE && -f /etc/redhat-release ]]; then

    release_num=`cat /etc/redhat-release | cut -d " " -f 7 | cut -d "." -f 1`

fi

if [[ -z $IS_SUSE && $release_num -ge 6 ]] ; then
     release=0
else
     release=1
fi

############################################ END ############################################################## 
HOST="127.0.0.1"
QUOTA_MON=`echo $QUOTA_MON | tr "[:lower:]" "[:upper:]" | tr -d "[:space:]"`

#Minimum NFS rpc protocol version supported is 2
min_nfsrpc_version=2

# Define portmap, rpc.statd, rpc.rquotad, rpc.mountd, nfsd, lockd, and
# their corresponding program number in pairs.
# Check if package is configured for NFSv4 version of protocol, then
# start only those daemons which are required by NFSv4 protocol.

  if [[ -z ${NFSv4_VIRTUAL_ROOT} ]]; then
     if [[ $release -eq 0 ]] || [[ ! -z "$IS_SUSE" ]] ; then
        HA_SERVER_PROC[0]="rpcbind"
        HA_SERVER[0]="/sbin/rpcbind"
     else
        HA_SERVER_PROC[0]="portmap"
        HA_SERVER[0]="/sbin/portmap"
     fi
     HA_SERVER_PROG[0]="100000"
     HA_SERVER_VER[0]=2


  HA_SERVER_PROC[1]="rpc.mountd"
  HA_SERVER[1]="/usr/sbin/rpc.mountd"
  HA_SERVER_PROG[1]="100005"
  HA_SERVER_VER[1]=3

  HA_SERVER_PROC[2]="nfsd"
  HA_SERVER[2]="/usr/sbin/rpc.nfsd"
  HA_SERVER_PROG[2]="100003"

  #Setting the HA_SERVER_VER for rpc.nfsd based on the availability
  #of the NFS RPC protocol version. This is required because
  #in RHEL7 and SLES12 by default version 2 is not enabled.
  if rpcinfo -u $HOST ${HA_SERVER_PROG[2]} $min_nfsrpc_version > /dev/null 2>&1
  then
    HA_SERVER_VER[2]=2
  else
    HA_SERVER_VER[2]=3
  fi

  #If it is RH5 system and not sles, nor RH6 and above.
  if [[ -z "${IS_SUSE}" ]] && [[ $release -eq 1 ]]; then
      HA_SERVER_PROC[3]="lockd"
      HA_SERVER[3]="/sbin/rpc.lockd"
      HA_SERVER_PROG[3]="100021"
      HA_SERVER_VER[3]=3

      HA_SERVER_PROC[4]="rpc.statd"
      HA_SERVER[4]="/sbin/rpc.statd"
      HA_SERVER_PROG[4]="100024"
      HA_SERVER_VER[4]=1


      if [[ $QUOTA_MON == "YES" ]]; then
        HA_SERVER_PROC[5]="rpc.rquotad"
        HA_SERVER[5]="/usr/sbin/rpc.rquotad"
        HA_SERVER_PROG[5]="100011"
        HA_SERVER_VER[5]=2
      fi
  else
      #If it is RH6 and above system or SLES 11 and above system
      HA_SERVER_PROC[3]="rpc.statd"
      HA_SERVER[3]="/sbin/rpc.statd"
      HA_SERVER_PROG[3]="100024"
      HA_SERVER_VER[3]=1

      if [[ $QUOTA_MON == "YES" ]]; then
         HA_SERVER_PROC[4]="rpc.rquotad"
         HA_SERVER[4]="/usr/sbin/rpc.rquotad"
         HA_SERVER_PROG[4]="100011"
         HA_SERVER_VER[4]=2
      fi
  fi

else    # NFSv4 is being used so make an array of mountd and nfsd.

  HA_SERVER_PROC[0]="rpc.mountd"
  HA_SERVER[0]="/usr/sbin/rpc.mountd"
  HA_SERVER_PROG[0]="100005"
  HA_SERVER_VER[0]=3

  HA_SERVER_PROC[1]="nfsd"
  HA_SERVER[1]="/usr/sbin/rpc.nfsd"
  HA_SERVER_PROG[1]="100003"
  HA_SERVER_VER[1]=4

  if [[ ! -z "${IS_SUSE}" ]]; then
     HA_SERVER_PROC[2]="rpcbind"
     HA_SERVER[2]="/sbin/rpcbind"
     HA_SERVER_PROG[2]="100000"
     HA_SERVER_VER[2]=2
  fi

fi


# This function checks if rpc.nfsd service should be  
# accessed via TCP in case of NFS V4 and the OS version
# >=15.0 in case of SLES.
# Change is now made for SLES12 as sles 12 updates also stopped support for udp.
# Note: This code block has been modified for RH >= 8.0 
# to support TCP for nfsd service.
function check_if_tcp_required_for_nfsd
{
	typeset nfs_program=$1
	typeset nfs_version=$2
	typeset -i ret_udp=1
        if [[ $nfs_program -eq 100003 ]];
        then
            os_version=$(check_for_os_version)

            if [[ "$os_version" != 0 ]];then

                 if [[ -n "$IS_SUSE" && $nfs_version -eq 4 ]];then

                     ret_udp=0

                 elif [[ -f /etc/redhat-release ]];then

                     ret_udp=`echo "$os_version" | awk '{if ($os_version >= 8.0) print "0"; else print "1"}'`
                 fi
            fi
        fi
	return $ret_udp
}

